/*============================================================================
  File:     InstantInitialization.sql

  Summary:  This script shows the differences in file creation with and without
			instant initialization.

  Date:     May 2006

  SQL Server Version: 9.00.2047.00 (SP1)
------------------------------------------------------------------------------
  Copyright (C) 2005-2006 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos, lectures and 
  resources created by Kimberly L. Tripp, President/Founder - SQLskills.com.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
CREATE DATABASE TestWithZeroInitialization
	ON PRIMARY 
	(NAME = N'TestWithZeroInitializationData', 
	FILENAME = N'C:\Program Files (x86)\Microsoft SQL Server\mssql.1\mssql\data\TestWithZeroInitializationData.MDF' , 
	SIZE = 20000,
	FILEGROWTH = 250,
	MAXSIZE = 30000)
LOG ON 
	(NAME = N'TestWithZeroInitializationLog', 
	FILENAME = N'C:\Program Files (x86)\Microsoft SQL Server\mssql.1\mssql\data\TestWithZeroInitializationLog.LDF' , 
	SIZE = 2, 
	FILEGROWTH = 10,
	MAXSIZE = 1000)
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go

-- Keep track of time here: 14:02 --842453 ms

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
ALTER DATABASE TestWithZeroInitialization
	MODIFY FILE (NAME = N'TestWithZeroInitializationData', SIZE = 30000)
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]

-- Keep track of time here: 7:01

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
RESTORE DATABASE [TestWithZeroInitialization] 
	FROM  DISK = N'C:\Program Files (x86)\Microsoft SQL Server\MSSQL.1\MSSQL\Backup\TestWithZeroInitialization.bak' 
	WITH  FILE = 1,  NOUNLOAD,  STATS = 10
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go

-- Keep track of time here: 21:07 -- 1267076 ms

-- LOAD it up with data

CREATE TABLE RowPerPage
(
	RowPerPageID	int	identity,
	RowPerPageTime	datetime	default getdate(),
	RowPerPageChar	char(4500)
)
go

-- insert ~1.4 million rows at 8K per row = ~10GB

ALTER DATABASE TestWithZeroInitialization
SET RECOVERY SIMPLE
go

SET NOCOUNT ON
go

DECLARE @Counter	int
SELECT @Counter = 0
WHILE @Counter < 1373692
BEGIN
	INSERT RowPerPage DEFAULT VALUES
	SELECT @Counter = @Counter + 1
END
go

SELECT count(*) FROM RowPerPage -- 1373691 rows
go

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
BACKUP DATABASE [TestWithZeroInitialization] 
	TO DISK = N'C:\Program Files (x86)\Microsoft SQL Server\MSSQL.1\MSSQL\Backup\TestWithZeroInitialization_10GBData.bak' 
	WITH INIT,  STATS = 10
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go

USE master
go

DROP DATABASE [TestWithZeroInitialization]
-- NOTE: In production, NEVER drop the database before restoring. SQL Server will use the already created
-- file and skip file creation altogether.
go

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
RESTORE DATABASE [TestWithZeroInitialization] 
	FROM  DISK = N'C:\Program Files (x86)\Microsoft SQL Server\MSSQL.1\MSSQL\Backup\TestWithZeroInitialization_10GBData.bak' 
	WITH  FILE = 1,  NOUNLOAD,  STATS = 10
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go

-- Keep track of time here: 38:28 -- 2308860 ms

--Grant the "Perform Volumn Maintenance Tasks" for the instance GROUP
-- Use Administrative Tools, Local Security Policy
--		Security Settings
--			Local Policies
--				User Rights Assignment

-- In this case:
--	For a default instance: SQLServer2005MSSQLUser$servername$MSSQLSERVER
--	For a named instance: SQLServer2005MSSQLUser$servername$instancename

--Then - you must stop and restart SQL Server

SELECT @@version
go

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
CREATE DATABASE TestInstantInitialization
	ON PRIMARY 
	(NAME = N'TestInstantInitializationData', 
	FILENAME = N'C:\Program Files (x86)\Microsoft SQL Server\mssql.1\mssql\data\TestInstantInitializationData.MDF' , 
	SIZE = 20000,
	FILEGROWTH = 250,
	MAXSIZE = 30000)
LOG ON 
	(NAME = N'TestInstantInitializationLog', 
	FILENAME = N'C:\Program Files (x86)\Microsoft SQL Server\mssql.1\mssql\data\TestInstantInitializationLog.LDF' , 
	SIZE = 2, 
	FILEGROWTH = 10,
	MAXSIZE = 1000)
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme Without Zero Initilization]
go
-- Keep track of time here: 01 --1343 ms

DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
ALTER DATABASE TestInstantInitialization
	MODIFY FILE (NAME = N'TestInstantInitializationData', SIZE = 30000)
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go
-- Keep track of time here: 00 --406 ms

DROP DATABASE TestInstantInitialization
go
DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
RESTORE DATABASE [TestInstantInitialization] 
	FROM  DISK = N'C:\Program Files (x86)\Microsoft SQL Server\MSSQL.1\MSSQL\Backup\TestWithZeroInitialization.bak' 
	WITH  FILE = 1,  NOUNLOAD,  STATS = 10, REPLACE
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go
-- Keep track of time here: 00 --5046 ms

DROP DATABASE TestInstantInitialization
go
DECLARE @StartTime	datetime
SELECT @StartTime = getdate()
RESTORE DATABASE [TestInstantInitialization] 
	FROM  DISK = N'C:\Program Files (x86)\Microsoft SQL Server\MSSQL.1\MSSQL\Backup\TestWithZeroInitialization_10GBData.bak' 
	WITH  FILE = 1,  NOUNLOAD,  STATS = 10, REPLACE
SELECT datediff(ms, @StartTime, getdate()) AS [TotalTIme With Zero Initilization]
go
-- Keep track of time here: 19:42 --1182716 ms

-- Clean up the test databases to free up space

-- DROP DATABASE TestWithZeroInitialization
-- DROP DATABASE TestInstantInitialization